/*
   Mini XML lib PLUS for C++

   Test for iterators

   Author: Giancarlo Niccolai <gian@niccolai.ws>

   $Id: testiter.cpp,v 1.3 2004/01/11 13:32:17 jonnymind Exp $
*/

#include <mxml_document.h>
#include <mxml_error.h>
#include <mxml.h>
#include <iostream>

#ifdef __WIN32__
#include <windows.h>
#else
#include <sys/time.h>
#endif

int main( int argc, char * argv[] )
{

   std::cout << "Please insert a valid XML document" << std::endl;

   #ifdef __WIN32__
   SYSTEMTIME st_start, st_end;
   GetSystemTime( &st_start );
   #else
   struct timeval tv_start, tv_end;
   gettimeofday(&tv_start, NULL);
   #endif
   
   try {
      std::cin.unsetf( std::ios_base::skipws );
      MXML::Document document( std::cin, MXML_STYLE_INDENT | MXML_STYLE_THREESPACES );
      MXML::Node *root = document.root();
      std::cout << "Flat node traversal of level 1" << std::endl;
      MXML::Node::const_iterator iter = root->const_begin();
      while ( iter != root->const_end() ) {
         std::cout << iter->name() <<"||" << iter->data() << std::endl;
         iter++;
      }

      std::cout << "Deep node traversal of level 1" << std::endl;
      MXML::Node::deep_iterator diter = root->deep_begin();
      while ( diter != root->end() ) {
         std::cout << diter->depth() << " | " << diter->path() << " | "
            << diter->data() << std::endl;
         diter++;
      }

      if ( argc > 1 ) {
         MXML::Node::find_iterator found;
         std::cout << std::endl << "-------------------------------------"<< std::endl <<
            "Finding node named: " << argv[1];
         if ( argc > 2 ) {
            std::cout << " with attribute " << argv[2];
            if ( argc > 3 ) {
               std::cout << " set to value " << argv[3];
               if ( argc > 4 ) {
                  std::cout << " containing data " << argv[4];
                  found = root->find( argv[1], argv[2], argv[3], argv[4] );
               }
               else
                  found = root->find( argv[1], argv[2], argv[3]);
            }
            else
               found = root->find( argv[1], argv[2] );
         }
         else
            found = root->find( argv[1] );

         std::cout << std::endl;
         while( found != root->end() ) {
            std::cout << found->path() << " | " << found->data() << std::endl;
            found++;
         }
      }
   }
   catch( MXML::Error &er )
   {
      std::cout << std::endl << er << std::endl;
   }

   #ifdef __WIN32__
   GetSystemTime( &st_end );
   std::cerr << std::endl << "Processed in " << (double)
      ( (st_end.wSecond - st_start.wSecond) * 1000 +
        (st_end.wMilliseconds - st_start.wMilliseconds ) ) 
      << " msecs" << std::endl;
   
   #else
   gettimeofday(&tv_end, NULL);
   std::cerr << std::endl << "Processed in " << (double)
      ( (tv_end.tv_sec - tv_start.tv_sec) * 1000 +
        (tv_end.tv_usec - tv_start.tv_usec ) /1000.0 ) << " msecs" << std::endl;
   #endif
   
   return 0;
}
